package com.vp.plugin.sample.zoomimage.actions;

import java.awt.*;

import com.vp.plugin.*;
import com.vp.plugin.action.*;
import com.vp.plugin.diagram.*;
import com.vp.plugin.model.*;

public class DemoActionController implements VPActionController {
	
	@Override
	public void performAction(VPAction aAction) {
		
		IProject lProject = ApplicationManager.instance().getProjectManager().getProject();
		
		/*
		 * simply this demo, 
		 * 
		 * assume there is only 1 UCD, shows its "Main" & "Sub" use cases
		 * 
		 * and 
		 * 
		 * assume there is only 1 Report Diagram.
		 * PS: since if the ReportDiagram is already opened. The changes inside the IOOTemplate cannot have any effects to the report diagram.
		 * So, here, won't get the report diagram from 'active' diagram.
		 */ 
		Integer lMinX = null; // the top/left of the diagram
		Integer lMinY = null;
		int lX1 = 0; // x/y of "Main"
		int lY1 = 0;
		int lX2 = 0; // x+width/y+height of "Sub"
		int lY2 = 0;
		IUseCaseDiagramUIModel lUCD = null; // assume won't be null
		IReportDiagramUIModel lReportDiagram = null; // assume won't be null
		{
			for (IDiagramUIModel lDiagram : lProject.toDiagramArray()) {
				if (lDiagram instanceof IUseCaseDiagramUIModel) {
					lUCD = (IUseCaseDiagramUIModel) lDiagram;
					
					for (IShapeUIModel lShape : lUCD.toShapeUIModelArray()) {
						if (lShape.getModelElement() instanceof IUseCase) {
							if ("Main".equals(lShape.getModelElement().getNickname())) {
								lX1 = lShape.getX();
								lY1 = lShape.getY();
							}
							else if ("Sub".equals(lShape.getModelElement().getNickname())) {
								lX2 = lShape.getX()+lShape.getWidth();
								lY2 = lShape.getY()+lShape.getHeight();
							}
						}
						
						if (lMinX == null) {
							lMinX = lShape.getX();
						}
						else {
							lMinX = Math.min(lShape.getX(), lMinX);
						}
						if (lMinY == null) {
							lMinY = lShape.getY();
						}
						else {
							lMinY = Math.min(lShape.getY(), lMinY);
						}
					}
				}
				else if (lDiagram instanceof IReportDiagramUIModel) {
					lReportDiagram = (IReportDiagramUIModel) lDiagram;
				}
			}
		}
		
		
		Rectangle lFocusedBounds = new Rectangle(lX1-lMinX, lY1-lMinY, lX2-lX1, lY2-lY1);
		// depends on different shapes showing on the diagram, there are different 'gap' showing on the diagram image. Need hardcode it.
		lFocusedBounds.x += 15;
		lFocusedBounds.y += 2;
		
		// get the 'Details' model element of the Report.
		IReportDiagramDetails lDetails = (IReportDiagramDetails) lProject.getModelElementByAddress(lReportDiagram.getGenericXMLElementAddress());
		// the id of the UCD, used to find out which RDImage is showing this UCD.
		String lUcdId = lUCD.getId();
		
		// assume this report already have OO template which contains the IRDImage of a diagram
		IModelElement[] lChildren = lDetails.toChildArray();
		if (lChildren != null) {
			for (IModelElement lChild : lChildren) { // the contents of the document
				focusImage(lUcdId, lFocusedBounds, lChild);
			}
		}
		
		// open the Report Diagram to check the result
		ApplicationManager.instance().getDiagramManager().openDiagram(lReportDiagram);
	}
	private void focusImage(String aUcdId, Rectangle aFocusedBounds, IModelElement aContent) {
		
		if (aContent instanceof IRDImage) {
			IRDImage lImage = (IRDImage) aContent;
			
			if (aUcdId.equals(lImage.getDiagramId())) {
				// this RDImage is showing the UCD's image. Zoom it now.
				
				double lScale = lImage.getScale();
				
				// scale to 100% (original should be 75%)
				lScale = 1;
				lImage.setScale(lScale);
				
				int lFocusedWidth = (int) (aFocusedBounds.width*lScale);
				int lFocusedHeight = (int) (aFocusedBounds.height*lScale);
				int lViewPositionX = aFocusedBounds.x*-1;
				int lViewPositionY = aFocusedBounds.y*-1;
				
				int lOriginalImageWidth = lImage.getImageBoundsWidth();
				
				lImage.setViewPositionX(lViewPositionX);
				lImage.setViewPositionY(lViewPositionY);
				lImage.setImageBoundsWidth(lFocusedWidth);
				lImage.setImageBoundsHeight(lFocusedHeight);
				
				int lPageWidth = lOriginalImageWidth + (lImage.getImageBoundsX()*2) + 10/*10px border of the image*/;
				lImage.setImageBoundsX((lPageWidth-lFocusedWidth)/2);
				
				return;
			}
			
		}
		else {
			// try its children
			IModelElement[] lChildren = aContent.toChildArray();
			if (lChildren != null) {
				for (IModelElement lChild : lChildren) {
					focusImage(aUcdId, aFocusedBounds, lChild);
				}
			}
		}
	}
	
	@Override
	public void update(VPAction aArg0) {
	}

}